@@ -27,6 +27,7 @@ from .compat import (
     stream_reader_factory,
     merge_params,
     normalize_url,
+    RequestInfo,
 )
 
 
@@ -34,7 +35,7 @@ class CallbackResult:
 
     def __init__(self, method: str = hdrs.METH_GET,
                  status: int = 200,
-                 body: str = '',
+                 body: Union[str, bytes] = '',
                  content_type: str = 'application/json',
                  payload: Dict = None,
                  headers: Dict = None,
@@ -56,7 +57,7 @@ class RequestMatch(object):
     def __init__(self, url: Union[URL, str, Pattern],
                  method: str = hdrs.METH_GET,
                  status: int = 200,
-                 body: str = '',
+                 body: Union[str, bytes] = '',
                  payload: Dict = None,
                  exception: 'Exception' = None,
                  headers: Dict = None,
@@ -117,7 +118,7 @@ class RequestMatch(object):
                         method: str = hdrs.METH_GET,
                         request_headers: Dict = None,
                         status: int = 200,
-                        body: str = '',
+                        body: Union[str, bytes] = '',
                         content_type: str = 'application/json',
                         payload: Dict = None,
                         headers: Dict = None,
@@ -136,7 +137,7 @@ class RequestMatch(object):
             loop = Mock()
             loop.get_debug = Mock()
             loop.get_debug.return_value = True
-            kwargs['request_info'] = Mock(
+            kwargs['request_info'] = RequestInfo(
                 url=url,
                 method=method,
                 headers=CIMultiDictProxy(CIMultiDict(**request_headers)),
@@ -178,9 +179,6 @@ class RequestMatch(object):
     async def build_response(
         self, url: URL, **kwargs
     ) -> 'Union[ClientResponse, Exception]':
-        if self.exception is not None:
-            return self.exception
-
         if callable(self.callback):
             if asyncio.iscoroutinefunction(self.callback):
                 result = await self.callback(url, **kwargs)
@@ -188,6 +186,10 @@ class RequestMatch(object):
                 result = self.callback(url, **kwargs)
         else:
             result = None
+
+        if self.exception is not None:
+            return self.exception
+
         result = self if result is None else result
         resp = self._build_response(
             url=url,
@@ -288,7 +290,7 @@ class aioresponses(object):
 
     def add(self, url: 'Union[URL, str, Pattern]', method: str = hdrs.METH_GET,
             status: int = 200,
-            body: str = '',
+            body: Union[str, bytes] = '',
             exception: 'Exception' = None,
             content_type: str = 'application/json',
             payload: Dict = None,
